@nodelist = (
    perl,
    perltoc,
    perldelta,
    perlfaq,
    perlfaq1,
    perlfaq2,
    perlfaq3,
    perlfaq4,
    perlfaq5,
    perlfaq6,
    perlfaq7,
    perlfaq8,
    perlfaq9,
    perldata,
    perlsyn,
    perlop,
    perlre,
    perlrun,
    perlfunc,
    perlvar,
    perlsub,
    perlmod,
    perlmodlib,
    perlform,
    perllocale,
    perlref,
    perldsc,
    perllol,
    perltoot,
    perlobj,
    perltie,
    perlbot,
    perlipc,
    perldebug,
    perldiag,
    perlsec,
    perltrap,
    perlstyle,
    perlpod,
    perlbook,
    perlembed,
    perlapio,
    perlxs,
    perlxstut,
    perlguts,
    perlcall,
    indices
);
1;